@@ -15,7 +15,7 @@
    limitations under the License.
 */
 
-/* $Id: mod_rivet.h 1518826 2013-08-29 21:20:38Z mxmanghi $ */
+/* $Id: mod_rivet.h 1583202 2014-03-30 21:32:12Z mxmanghi $ */
 
 #ifndef MOD_RIVET_H
 #define MOD_RIVET_H 1
@@ -96,7 +96,7 @@ typedef struct _rivet_server_conf {
     apr_table_t *rivet_server_vars;
     apr_table_t *rivet_dir_vars;
     apr_table_t *rivet_user_vars;
-    char **objCacheList;		    /* Array of cached objects (for priority handling) */
+    char **objCacheList;		        /* Array of cached objects (for priority handling) */
     Tcl_HashTable *objCache;		    /* Objects cache - the key is the script name */
 
     Tcl_Channel *outchannel;		    /* stuff for buffering output */
@@ -105,15 +105,15 @@ typedef struct _rivet_server_conf {
 /* eventually we will transfer 'global' variables in here and 'de-globalize' them */
 
 typedef struct _rivet_interp_globals {
-    request_rec     *r;			    /* request rec */
-    TclWebRequest   *req;	            /* TclWeb API request */
-    Tcl_Namespace   *rivet_ns;              /* Rivet commands namespace */
-    int             page_aborting;	    /* set by abort_page. */
-					    /* to be reset by Rivet_SendContent */
-    Tcl_Obj*        abort_code;
+    request_rec     *r;			        /* request rec              */
+    TclWebRequest   *req;	            /* TclWeb API request       */
+    Tcl_Namespace   *rivet_ns;          /* Rivet commands namespace */
+    int             page_aborting;	    /* set by abort_page.       */
+    Tcl_Obj*        abort_code;			/* To be reset by Rivet_SendContent */
+    server_rec*     srec;               /* pointer to the current server rec obj */
 } rivet_interp_globals;
 
-int Rivet_ParseExecFile(TclWebRequest *req, char *filename, int toplevel);
+int Rivet_ParseExecFile   (TclWebRequest *req, char *filename, int toplevel);
 int Rivet_ParseExecString (TclWebRequest* req, Tcl_Obj* inbuf);
 
 rivet_server_conf *Rivet_GetConf(request_rec *r);
@@ -141,6 +141,8 @@ Tcl_Obj* Rivet_ReadConfParameter (  Tcl_Interp*         interp,
 Tcl_Obj* Rivet_CurrentConfDict (    Tcl_Interp*           interp,
                                     rivet_server_conf*    rivet_conf);
 
+Tcl_Obj* Rivet_CurrentServerRec (   Tcl_Interp*         interp, server_rec* s );
+
 /* error code set by command 'abort_page' */
 
 #define ABORTPAGE_CODE "ABORTPAGE"
